/****************************************************************	
* Name : Atul Nayak						*
* door.cxx : Methods of class door				*
****************************************************************/	     

#include "common.hxx"
#include "door.hxx"

//constructor - reads the door.txt file
door::door()
{ 
	int tmp_noOfVertices;
	float tmp_vertex1;
	float tmp_vertex2;
	float tmp_vertex3;
	float tmp_nx;
	float tmp_ny;
	float tmp_nz;

	FILE* fp1 = fopen(doorfile,"r");

	if (!fp1)
	{
		printf("Couldn't open file door.txt\n");
		exit(1);
	}
	fscanf(fp1,"%d\n", &noOfPolygons);
	fscanf(fp1,"\n");

	fscanf(fp1,"%f %f %f", &colors[0], &colors[1], &colors[2]);
	fscanf(fp1,"\n");

	for (int i = 0; i < noOfPolygons; i++)
	{
		fscanf(fp1, "%d", &tmp_noOfVertices);
		noOfVertices.push_back(tmp_noOfVertices);
		fscanf(fp1,"\n");

		// read in the normals 
		// store them in the normals vector
		fscanf(fp1, "%f %f %f", &tmp_nx, &tmp_ny, &tmp_nz);
		normals.push_back(tmp_nx);
		normals.push_back(tmp_ny);
		normals.push_back(tmp_nz);
		fscanf(fp1,"\n");

		for (int k =0 ; k < tmp_noOfVertices; k++)
		{
			fscanf(fp1, "%f %f %f", &tmp_vertex1, &tmp_vertex2, &tmp_vertex3);
			vertices.push_back(tmp_vertex1);
			vertices.push_back(tmp_vertex2);
			vertices.push_back(tmp_vertex3);
			fscanf(fp1,"\n");
		}
	}
	fclose(fp1);
	R = colors[0];
	G = colors[1];
	B = colors[2];

}

// rotate the door by 90 degrees if required
void door::setAngle()
{
	angle = 90.0;
}

// draw the Door at the position specified
void door::drawDoor(int x, int y)
{
	positionX = x;
	positionY = y;

	GLfloat textureVector[] = {0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f};

	glPushMatrix();
		glTranslatef((x*5.0)+2.5,(y*5.0)+2.5,7.0);
		glRotatef(angle, 0,0,1);	
//		glColor3f(colors[0], colors[1], colors[2]);

		// define the material properties	
		GLfloat mat_diffuse[] = {colors[0], colors[1], colors[2], 1.0};
		glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);

		int i =0;		// i keeps track of the no of coordinates
		int nCtr = 0;		// nCtr keeps track of the normal coordinates for  a polygon
		for (int j = 0; j < noOfPolygons; j++, nCtr += 3)
		{
			glBegin(GL_POLYGON);
				int g=0;	// g tracks no of corners in the polygon
				int tCtr = 0;	// Texture coordinate counter 
				for ( ; g < noOfVertices[j]; i+=3, g++, tCtr+=2)
				{	
					glNormal3f(normals[nCtr], normals[nCtr+1], normals[nCtr+2]);
					glTexCoord2f (textureVector[tCtr],textureVector[tCtr+1]);
					glVertex3f(vertices[i], vertices[i+1], vertices[i + 2]);
				}
			glEnd();
		}
	glPopMatrix();
	
}

//return positionX
int door::getPositionX()
{
	return positionX;
}

//return positionY
int door::getPositionY()
{
	return positionY;
}

// set the color to it's maximum intensity
void door::brightestColor()
{
	colors[0] = R;
	colors[1] = G;		
	colors[2] = B;		
}

// set the color to it's minimum intensity
void door::darkestColor()
{
	colors[0] = 0.0;
	colors[1] = 0.0;		
	colors[2] = 0.0;		
}

// set the color to an intermediate value specified
void door::changeColor(float difference)
{
	if (R > 0.0)
		colors[0] = difference;
	if (G > 0.0)
		colors[1] = difference;
	if (B > 0.0)
		colors[2] = difference;		
}

// destructor
door::~door()
{}
