/****************************************************************	
* Name : Atul Nayak						*
* flutter.cxx : Methods of class flutter			*
****************************************************************/	     

#include "common.hxx"
#include "flutter.hxx"

/* The path coordinates below trace the route that eacj butterfly object will */
/* take. 								      */	
int clockCtr = 0;
int path1[] = { 14, 22,
	    	14, 23,
	    	14, 24,
		14, 26,
		13, 27,
		13, 28,
		13, 29,
		13, 30,
		13, 31,
		14, 32,
		14, 33,
		14, 34,
		14, 35,
		13, 35};			

int path2[] = { 17, 14,
		18, 14,
		19, 14,
		20, 14,
		20, 13,
		20, 12,
		20, 11,
		20, 10,
		20, 9};

int path3[] = { 17, 30,
		18, 30,
		19, 30,
		19, 31,
		19, 32,
		19, 33,
		19, 34,
		19, 35};

// constructor - reads the flutter.txt file
flutter::flutter()
{ 
	int tmp_noOfVertices;
	float tmp_vertex1;
	float tmp_vertex2;
	float tmp_vertex3;
	float tmp_nx;
	float tmp_ny;
	float tmp_nz;

	FILE* fp1 = fopen(flutterfile,"r");

	if (!fp1)
	{
		printf("Couldn't open file flutter.txt\n");
		exit(1);
	}
	fscanf(fp1,"%d\n", &noOfPolygons);
	fscanf(fp1,"\n");

	fscanf(fp1,"%f %f %f", &colors[0], &colors[1], &colors[2]);
	fscanf(fp1,"\n");
	
	// i keeps track of the number of polygons
	for (int i = 0; i < noOfPolygons; i++)
	{
		fscanf(fp1, "%d", &tmp_noOfVertices);
		noOfVertices.push_back(tmp_noOfVertices);
		fscanf(fp1,"\n");

		// read in the normals 
		// store them in the normals vector
		fscanf(fp1, "%f" "%f" "%f", &tmp_nx, &tmp_ny, &tmp_nz);
		normals.push_back(tmp_nx);
		normals.push_back(tmp_ny);
		normals.push_back(tmp_nz);
		fscanf(fp1,"\n");

		// k tracks the no of vertices
		for (int k =0 ; k < tmp_noOfVertices; k++)
		{
			fscanf(fp1, "%f %f %f", &tmp_vertex1, &tmp_vertex2, &tmp_vertex3);
			vertices.push_back(tmp_vertex1);
			vertices.push_back(tmp_vertex2);
			vertices.push_back(tmp_vertex3);
			fscanf(fp1,"\n");
		}
	}
	fclose(fp1);
	ctr = 0;
	notFirstTime = FALSE;
	nextZ = 3.5;
	goUp = TRUE;
	goDown = FALSE;

}

// draw the flutter at the specified position
void flutter::drawFlutter(int x, int y, int number)
{
	if (!notFirstTime)
	{
		flutterNo = number;
		nextX = x;
		nextY = y;
	}
	positionX = x;
	positionY = y;

	// apply transformations	
	glPushMatrix();
		glTranslatef((nextX*5.0)+2.5,(nextY*5.0)+2.5,nextZ);	
		GLfloat mat_diffuse[] = {colors[0], colors[1], colors[2], 1.0};
		glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);

		int i =0;		// i keeps track of the no of coordinates
		int nCtr = 0;		// nCtr keeps track of the normal coordinates for  a polygon
		for (int j = 0; j < noOfPolygons; j++, nCtr += 3)
		{
			glBegin(GL_POLYGON);
				int g=0;	// g tracks no of corners in the polygon
				for ( ; g < noOfVertices[j]; i+=3, g++)
				{
					glNormal3f(normals[nCtr], normals[nCtr+1], normals[nCtr+2]);
					glVertex3f(vertices[i], vertices[i+1], vertices[i + 2]);
				}
			glEnd();
		}
		glutSolidSphere(0.5,20,20);	// draw flutter's head
	glPopMatrix();
	
}

//return positionX
int flutter::getPositionX()
{
	return positionX;
}

//return positionY
int flutter::getPositionY()
{
	return positionY;
}

//return nextX
int flutter::getNextX()
{
	return nextX;
}

//return nextY
int flutter::getNextY()
{
	return nextY;
}

// this routine moves the flutter object along one of three
// predetermined paths. The path ends into a teleporter which
// takes you to a separate part of the world where many orbs are located
void flutter::moveFlutter()
{

	if (flutterNo == 0)
	{
		clockCtr++;
		if (clockCtr == 10)
		{
			ctr+= 2;
			if (ctr > 26)
				ctr = 0;
			nextX = path1[ctr];
			nextY = path1[ctr+1];	
			system("/usr/sbin/sfplay bluejay2.au &");
			clockCtr = 0;
		}	
	}

	if (flutterNo == 1)
	{
		clockCtr++;
		if (clockCtr == 10)
		{
			ctr+= 2;
			if (ctr > 16)
				ctr = 0;
			nextX = path2[ctr];
			nextY = path2[ctr+1];	
			system("/usr/sbin/sfplay bluejay2.au &");
			clockCtr = 0;
		}	
	}

	if (flutterNo == 2)
	{
		clockCtr++;
		if (clockCtr == 10)
		{
			ctr+= 2;
			if (ctr > 14)
				ctr = 0;
			nextX = path3[ctr];
			nextY = path3[ctr+1];	
			system("/usr/sbin/sfplay bluejay2.au &");
			clockCtr = 0;
		}	
	}


	// this part of the routine moves the flutter object
	// along the z-axis to give it some animation
	if (nextZ > 4.0)
	{
		goDown = TRUE;
		goUp = FALSE;
	}

	if (nextZ < 2.5)
	{
		goUp = TRUE;
		goDown = FALSE;
	}
	
	if (goUp)
		nextZ += 0.1;

	if (goDown)
		nextZ -= 0.1;
	
	
}	

// set notFirstTime
void flutter::setNotFirstTime()
{
	notFirstTime = TRUE;
}


//destructor
flutter::~flutter()
{}
